#ifndef VARIOSCREEN_H
#define VARIOSCREEN_H

#include <Arduino.h>
#include <SPI.h>
#include <digit.h>

#define VARIOSCREEN_DISPLAY_THRESHOLD 0.65

class VarioScreen {
 public:
  VarioScreen(int8_t DC, int8_t CS, int8_t RST);
  void begin(uint8_t clockDiviser = SPI_CLOCK_DIV2, uint8_t contrast = 40, uint8_t bias = 0x04);
  void beginDisplay(uint8_t x, uint8_t y);
  void display(uint8_t displayByte);
  void endDisplay();
  void flush();
  void clear();

 private:
  void command(uint8_t c);
  int8_t _dc, _rst, _cs;
};


class ScreenDigit {

 public:
  ScreenDigit(VarioScreen& screen, uint8_t anchorX, uint8_t anchorY, uint8_t precision, boolean plusDisplay = false) : screen(screen), digit(precision, plusDisplay), anchorX(anchorX), anchorY(anchorY) { lastDisplayWidth = 0; lastDisplayValue = 10000000.0; }
  
  void display(double value);
  void display(int value, uint8_t maxchar, bool zero);

 private:
  void displayCharacter(const uint8_t* pointer, uint8_t width, uint8_t line);
  VarioScreen& screen;
  Digit digit;
  uint8_t anchorX, anchorY;
  uint8_t lastDisplayWidth;
  double lastDisplayValue;
};

class MSUnit {

 public :
  MSUnit(VarioScreen& screen) : screen(screen) { }
  void display(uint8_t posX, uint8_t posY);

 private :
  VarioScreen& screen;
};

class MUnit {

 public :
  MUnit(VarioScreen& screen) : screen(screen) { }
  void display(uint8_t posX, uint8_t posY);

 private :
  VarioScreen& screen;
};

class KMHUnit {

 public :
  KMHUnit(VarioScreen& screen) : screen(screen) { }
  void display(uint8_t posX, uint8_t posY);

 private :
  VarioScreen& screen;
};

class GRUnit {

 public :
  GRUnit(VarioScreen& screen) : screen(screen) { }
  void display(uint8_t posX, uint8_t posY);

 private :
  VarioScreen& screen;
};

class BATUnit {

 public :
  BATUnit(VarioScreen& screen) : screen(screen) { }
  void display(uint8_t posX, uint8_t posY);
  void display(uint8_t posX, uint8_t posY, double voltage);

 private :
  VarioScreen& screen;
};

class SATUnit {

 public :
  SATUnit(VarioScreen& screen) : screen(screen) { }
  void display(uint8_t posX, uint8_t posY);
  void display(uint8_t posX, uint8_t posY, uint8_t satelite);

 private :
  VarioScreen& screen;
};

class ScreenTime {

 public:
  ScreenTime(VarioScreen& screen, uint8_t anchorX, uint8_t anchorY) : screen(screen),   TimeHour(screen, anchorX, anchorY, 0, false), 
     TimeMinute(screen, anchorX+28, anchorY, 0, false), anchorX(anchorX), anchorY(anchorY) {}
  
  void display(unsigned long seconde);
  void display(char Str[5]);
  void displayTime();
  void displayDuration();

 private:
  void displayCharacter(const uint8_t* pointer, uint8_t width, uint8_t line);
  VarioScreen& screen;
  uint8_t anchorX, anchorY;
  ScreenDigit TimeMinute;
  ScreenDigit TimeHour;
  bool durationdisplay = false;       
};
#endif
